@@ -671,3 +671,216 @@ func TestFritschButlandErrors(t *testing.T) {
 		}
 	}
 }
+
+func TestPiecewiseCubicFitWithSecondDerivatives(t *testing.T) {
+	t.Parallel()
+	const tol = 1e-14
+	xs := []float64{-2, 0, 3}
+	ys := []float64{2.5, 1, 2.5}
+	d2ydx2s := []float64{1, 2, 3}
+	var pc PiecewiseCubic
+	pc.fitWithSecondDerivatives(xs, ys, d2ydx2s)
+	m := len(xs) - 1
+	if pc.lastY != ys[m] {
+		t.Errorf("Mismatch in lastY: got %v, want %g", pc.lastY, ys[m])
+	}
+	if !floats.Equal(pc.xs, xs) {
+		t.Errorf("Mismatch in xs: got %v, want %v", pc.xs, xs)
+	}
+	for i := 0; i < len(xs); i++ {
+		yHat := pc.Predict(xs[i])
+		if math.Abs(yHat-ys[i]) > tol {
+			t.Errorf("Mismatch in predicted Y[%d]: got %v, want %g", i, yHat, ys[i])
+		}
+		var d2ydx2Hat float64
+		if i < m {
+			d2ydx2Hat = 2 * pc.coeffs.At(i, 2)
+		} else {
+			d2ydx2Hat = 2*pc.coeffs.At(m-1, 2) + 6*pc.coeffs.At(m-1, 3)*(xs[m]-xs[m-1])
+		}
+		if math.Abs(d2ydx2Hat-d2ydx2s[i]) > tol {
+			t.Errorf("Mismatch in predicted d2Y/dX2[%d]: got %v, want %g", i, d2ydx2Hat, d2ydx2s[i])
+		}
+	}
+	// Test pc.lastDyDx without copying verbatim the calculation from the tested method:
+	lastDyDx := pc.PredictDerivative(xs[m] - tol/1000)
+	if math.Abs(lastDyDx-pc.lastDyDx) > tol {
+		t.Errorf("Mismatch in lastDxDy: got %v, want %g", pc.lastDyDx, lastDyDx)
+	}
+}
+
+func TestPiecewiseCubicFitWithSecondDerivativesErrors(t *testing.T) {
+	t.Parallel()
+	for _, test := range []struct {
+		xs, ys, d2ydx2s []float64
+	}{
+		{
+			xs:      []float64{0, 1, 2},
+			ys:      []float64{10, 20},
+			d2ydx2s: []float64{0, 0, 0},
+		},
+		{
+			xs:      []float64{0, 1, 1},
+			ys:      []float64{10, 20, 30},
+			d2ydx2s: []float64{0, 0, 0, 0},
+		},
+		{
+			xs:      []float64{0},
+			ys:      []float64{0},
+			d2ydx2s: []float64{0},
+		},
+		{
+			xs:      []float64{0, 1, 1},
+			ys:      []float64{10, 20, 10},
+			d2ydx2s: []float64{0, 0, 0},
+		},
+	} {
+		var pc PiecewiseCubic
+		if !panics(func() { pc.fitWithSecondDerivatives(test.xs, test.ys, test.d2ydx2s) }) {
+			t.Errorf("expected panic for xs: %v, ys: %v and d2ydx2s: %v", test.xs, test.ys, test.d2ydx2s)
+		}
+	}
+}
+
+func TestMakeCubicSplineSecondDerivativeEquations(t *testing.T) {
+	t.Parallel()
+	const tol = 1e-15
+	xs := []float64{-1, 0, 2}
+	ys := []float64{2, 0, 2}
+	n := len(xs)
+	a := mat.NewTridiag(n, nil, nil, nil)
+	b := mat.NewVecDense(n, nil)
+	makeCubicSplineSecondDerivativeEquations(a, b, xs, ys)
+	if b.Len() != n {
+		t.Errorf("Mismatch in b size: got %v, want %d", b.Len(), n)
+	}
+	r, c := a.Dims()
+	if r != n || c != n {
+		t.Errorf("Mismatch in A size: got %d x %d, want %d x %d", r, c, n, n)
+	}
+	expectedB := mat.NewVecDense(3, []float64{0, 3, 0})
+	var diffB mat.VecDense
+	diffB.SubVec(b, expectedB)
+	if diffB.Norm(math.Inf(1)) > tol {
+		t.Errorf("Mismatch in b values: got %v, want %v", b, expectedB)
+	}
+	expectedA := mat.NewDense(3, 3, []float64{0, 0, 0, 1 / 6., 1, 2 / 6., 0, 0, 0})
+	var diffA mat.Dense
+	diffA.Sub(a, expectedA)
+	if diffA.Norm(math.Inf(1)) > tol {
+		t.Errorf("Mismatch in A values: got %v, want %v", a, expectedA)
+	}
+}
+
+func TestNaturalCubicFit(t *testing.T) {
+	t.Parallel()
+	xs := []float64{-1, 0, 2, 3.5}
+	ys := []float64{2, 0, 2, 1.5}
+	var nc NaturalCubic
+	err := nc.Fit(xs, ys)
+	if err != nil {
+		t.Errorf("Error when fitting NaturalCubic: %v", err)
+	}
+	testXs := []float64{-1, -0.99, -0.5, 0, 0.5, 1, 1.5, 2, 2.5, 3, 3.49, 3.5}
+	// From scipy.interpolate.CubicSpline:
+	want := []float64{
+		2.0,
+		1.9737725526315788,
+		0.7664473684210527,
+		0.0,
+		0.027960526315789477,
+		0.6184210526315789,
+		1.3996710526315788,
+		2.0,
+		2.1403508771929824,
+		1.9122807017543857,
+		1.508859403508772,
+		1.5}
+	for i := 0; i < len(testXs); i++ {
+		got := nc.Predict(testXs[i])
+		if math.Abs(got-want[i]) > 1e-14 {
+			t.Errorf("Mismatch in predicted Y value for x = %g: got %v, want %g", testXs[i], got, want[i])
+		}
+		got = nc.PredictDerivative(testXs[i])
+		want := discrDerivPredict(&nc, xs[0], xs[len(xs)-1], testXs[i], 1e-9)
+		if math.Abs(got-want) > 1e-6 {
+			t.Errorf("Mismatch in predicted dY/dX value for x = %g: got %v, want %g", testXs[i], got, want)
+		}
+	}
+}
+
+func TestClampedCubicFit(t *testing.T) {
+	t.Parallel()
+	xs := []float64{-1, 0, 2, 3.5}
+	ys := []float64{2, 0, 2, 1.5}
+	var cc ClampedCubic
+	err := cc.Fit(xs, ys)
+	if err != nil {
+		t.Errorf("Error when fitting ClampedCubic: %v", err)
+	}
+	testXs := []float64{-1, -0.99, -0.5, 0, 0.5, 1, 1.5, 2, 2.5, 3, 3.49, 3.5}
+	// From scipy.interpolate.CubicSpline:
+	want := []float64{
+		2.0,
+		1.999564111111111,
+		1.2021604938271606,
+		0.0,
+		-0.20833333333333343,
+		0.41975308641975284,
+		1.337962962962962,
+		2.0,
+		2.026748971193416,
+		1.7078189300411522,
+		1.5001129711934151,
+		1.4999999999999996,
+	}
+	for i := 0; i < len(testXs); i++ {
+		got := cc.Predict(testXs[i])
+		if math.Abs(got-want[i]) > 1e-14 {
+			t.Errorf("Mismatch in predicted Y value for x = %g: got %v, want %g", testXs[i], got, want[i])
+		}
+		got = cc.PredictDerivative(testXs[i])
+		want := discrDerivPredict(&cc, xs[0], xs[len(xs)-1], testXs[i], 1e-9)
+		if math.Abs(got-want) > 1e-6 {
+			t.Errorf("Mismatch in predicted dY/dX value for x = %g: got %v, want %g", testXs[i], got, want)
+		}
+	}
+}
+
+func TestNotAKnotCubicFit(t *testing.T) {
+	t.Parallel()
+	xs := []float64{-1, 0, 2, 3.5}
+	ys := []float64{2, 0, 2, 1.5}
+	var nak NotAKnotCubic
+	err := nak.Fit(xs, ys)
+	if err != nil {
+		t.Errorf("Error when fitting NotAKnotCubic: %v", err)
+	}
+	testXs := []float64{-1, -0.99, -0.5, 0, 0.5, 1, 1.5, 2, 2.5, 3, 3.49, 3.5}
+	// From scipy.interpolate.CubicSpline:
+	want := []float64{
+		2.0,
+		1.961016095238095,
+		0.5582010582010581,
+		0.0,
+		0.09523809523809526,
+		0.6137566137566137,
+		1.3253968253968258,
+		2.0,
+		2.407407407407407,
+		2.3174603174603177,
+		1.5249675026455023,
+		1.5000000000000004,
+	}
+	for i := 0; i < len(testXs); i++ {
+		got := nak.Predict(testXs[i])
+		if math.Abs(got-want[i]) > 1e-14 {
+			t.Errorf("Mismatch in predicted Y value for x = %g: got %v, want %g", testXs[i], got, want[i])
+		}
+		got = nak.PredictDerivative(testXs[i])
+		want := discrDerivPredict(&nak, xs[0], xs[len(xs)-1], testXs[i], 1e-9)
+		if math.Abs(got-want) > 1e-6 {
+			t.Errorf("Mismatch in predicted dY/dX value for x = %g: got %v, want %g", testXs[i], got, want)
+		}
+	}
+}
